const gulp = require('gulp');
const less = require('gulp-less');
const cleanCss = require('gulp-clean-css');
const connect = require('gulp-connect');
const htmlbeautify = require('gulp-html-beautify');
const clean = require('gulp-clean');
const fileInclude = require('gulp-file-include');
const babel = require('gulp-babel');
const webpack = require('webpack-stream');

const paths = {
  html: './html/**/*.html',
  css: './html/css/**/*.css',
  img: './html/images/**',
  js: './html/js/**',
  less: './html/less/**/*.less',
  es6: './html/es6/main.js'
}

// 清理目标文件夹/文件
gulp.task('clean', function() {
  return gulp.src('dist')
    .pipe(clean({
      force: true
    }))
});

// 格式化 html
gulp.task('htmlbeautify', function() {
  return gulp.src('dist/**/*.html')
    .pipe(htmlbeautify({
      indent_size: 2,
      indent_char: ' '
    }))
    .pipe(gulp.dest('dist/'))
});

// 复制html
gulp.task('copyHtml', function() {
  return gulp.src(paths.html)
    .pipe(fileInclude({
      prefix: '@@',
      basepath: '@file'
    }))
    .pipe(gulp.dest('dist/'))
    .pipe(connect.reload());
});

// 复制css
gulp.task('copyCss', function() {
  return gulp.src(paths.css)
    .pipe(gulp.dest('dist/css/'))
    .pipe(connect.reload());
});

// 复制img
gulp.task('copyImg', function() {
  return gulp.src(paths.img)
    .pipe(gulp.dest('dist/images/'))
    .pipe(connect.reload());
});

// 复制js
gulp.task('copyJs', function() {
  return gulp.src(paths.js)
    .pipe(gulp.dest('dist/js/'))
    .pipe(connect.reload());
});

// less css 预处理
gulp.task('less', function() {
  return gulp.src(paths.less)
    .pipe(less())
    .pipe(cleanCss())
    .pipe(gulp.dest('dist/css/'))
    .pipe(connect.reload());
});

// babel
gulp.task('babel', function() {
  return gulp.src(paths.es6)
    .pipe(babel({
      presets: ['@babel/preset-env']
    }))
    .pipe(webpack({
      output: {
        filename: 'main.js'
      }
    }))
    .pipe(gulp.dest('dist/js/'))
    .pipe(connect.reload());
})

// connect任务
gulp.task('connect', function () {
  return connect.server({
    root: 'dist',
    livereload: true
  });
});

// watch任务
gulp.task('watch', function () {
  gulp.watch(paths.html, gulp.series('copyHtml'));
  gulp.watch(paths.css, gulp.series('copyCss'));
  gulp.watch(paths.img, gulp.series('copyImg'));
  gulp.watch(paths.js, gulp.series('copyJs'));
  gulp.watch(paths.less, gulp.series('less'));
  gulp.watch(paths.es6, gulp.series('babel'));
});

gulp.task('copy', gulp.parallel('copyHtml', 'copyCss', 'copyImg', 'copyJs'));
gulp.task('default', gulp.series('copy', 'babel', 'less', 'connect', 'watch'));